'*****************************************************************************
'                                                                            *
'  Cube.bas                                                                  *
'                                                                            *
'  This program draws a cube in 3D world space and allows the user to move   *
'  and rotate the cube through keyboard controls. Each of the six cube faces *
'  is a different color.                                                     *
'                                                                            *
'*****************************************************************************

#COMPILE EXE "Cube.exe"

#INCLUDE "FGWin.inc"

%vbWidth = 640
%vbHeight = 480

TYPE POINT3D
  X AS DOUBLE
  Y AS DOUBLE
  Z AS DOUBLE
END TYPE

GLOBAL Colors() AS LONG

' six faces of a 40x40x40 cube, defined in object coordinates
GLOBAL Faces() AS POINT3D

'------------------------------------------------------------------------------

FUNCTION WINMAIN (BYVAL hInstance     AS LONG, _
                  BYVAL hPrevInstance AS LONG, _
                  lpCmdLine           AS ASCIIZ PTR, _
                  BYVAL iCmdShow      AS LONG) AS LONG

  LOCAL Msg         AS tagMsg
  LOCAL wndclass    AS WndClassEx
  LOCAL szClassName AS ASCIIZ * 80
  LOCAL hWnd        AS LONG

  DIM Colors(1:6)
  DIM Faces(1:4,1:6)

  szClassName            = "FGcube"

  wndclass.cbSize        = SIZEOF(WndClass)
  wndclass.style         = %CS_HREDRAW OR %CS_VREDRAW OR %CS_OWNDC
  wndclass.lpfnWndProc   = CODEPTR(WindowProc)
  wndclass.cbClsExtra    = 0
  wndclass.cbWndExtra    = 0
  wndclass.hInstance     = hInstance
  wndclass.hIcon         = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  wndclass.hCursor       = LoadCursor(%NULL, BYVAL %IDC_ARROW)
  wndclass.hbrBackground = %NULL
  wndclass.lpszMenuName  = %NULL
  wndclass.lpszClassName = VARPTR(szClassName)
  wndclass.hIconSm       = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  RegisterClassEx wndclass

  hWnd = CreateWindow( _
    szClassName, _               ' window class name
    "Single 3D Cube", _          ' window caption
    %WS_OVERLAPPEDWINDOW, _      ' window style
    %CW_USEDEFAULT, _            ' initial x position
    %CW_USEDEFAULT, _            ' initial y position
    %CW_USEDEFAULT, _            ' initial x size
    %CW_USEDEFAULT, _            ' initial y size
    %NULL, _                     ' parent window handle
    %NULL, _                     ' window menu handle
    hInstance, _                 ' program instance handle
    BYVAL %NULL)                 ' creation parameters

  ShowWindow hWnd, iCmdShow
  UpdateWindow hWnd

  ' The message loop processes entries placed in the message queue.
  ' When no message is ready, call CheckForMovement() to check if
  ' we want to perform movement.

  WHILE %TRUE
    IF PeekMessage(Msg, %NULL, 0, 0, %PM_REMOVE) THEN
      IF Msg.message = %WM_QUIT THEN
        EXIT LOOP
      ELSE
        TranslateMessage Msg
        DispatchMessage Msg
      END IF
    ELSE
      CALL CheckForMovement
    END IF
  WEND

  FUNCTION = Msg.wParam

END FUNCTION

'------------------------------------------------------------------------------

GLOBAL hDC AS LONG
GLOBAL hPal AS LONG
GLOBAL hVB AS LONG
GLOBAL cxClient AS LONG, cyClient AS LONG
GLOBAL vbDepth AS LONG

GLOBAL Redraw AS LONG
GLOBAL xAngle AS LONG, yAngle AS LONG, zAngle AS LONG
GLOBAL xWorld AS DOUBLE, yWorld AS DOUBLE, zWorld AS DOUBLE

FUNCTION WindowProc (BYVAL hWnd AS LONG, BYVAL iMsg AS LONG, _
                     BYVAL wParam AS LONG, BYVAL lParam AS LONG) EXPORT AS LONG

  LOCAL ps AS PAINTSTRUCT
  LOCAL bpp AS LONG

  SELECT CASE iMsg

    CASE %WM_CREATE
      hDC = GetDC(hWnd)
      fg_setdc hDC
      hPal = fg_defpal()
      fg_realize hPal

      fg_vbinit
      vbDepth = fg_colors()
      fg_vbdepth vbDepth
      hVB = fg_vballoc(%vbWidth, %vbHeight)
      fg_vbopen hVB
      fg_vbcolors
      fg_setcolor -1
      fg_fillpage

      fg_3Dviewport 0, %vbWidth-1, 0, %vbHeight-1, 0.5
      fg_3Drenderstate %FG_ZCLIP
      xAngle = 0
      yAngle = 0
      zAngle = 0
      xWorld = 0.0
      yWorld = 0.0
      zWorld = 100.0
      Redraw = %TRUE

      CALL InitGlobals

      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_PAINT
      BeginPaint hWnd, ps
      fg_vbscale 0, %vbWidth-1, 0, %vbHeight-1, 0, cxClient-1, 0, cyClient-1
      EndPaint hWnd, ps
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SETFOCUS
      fg_realize hPal
      InvalidateRect hWnd, BYVAL %NULL, %TRUE
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SIZE
      cxClient = LOWRD(lParam)
      cyClient = HIWRD(lParam)
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_DESTROY
      fg_vbclose
      fg_vbfree hVB
      fg_vbfin
      DeleteObject hPal
      ReleaseDC hWnd, hDC
      PostQuitMessage 0
      FUNCTION = 0
      EXIT FUNCTION

  END SELECT

  FUNCTION = DefWindowProc(hWnd, iMsg, wParam, lParam)

END FUNCTION

'*****************************************************************************
'                                                                            *
'  CheckForMovement()                                                        *
'                                                                            *
'  The CheckForMovement() function checks for key presses that control the   *
'  cube's movement, and if required redraws the cube at its new position and *
'  orientation. It is called from the WinMain() message loop when there are  *
'  no messages waiting.                                                      *
'                                                                            *
'*****************************************************************************

SUB CheckForMovement()

  LOCAL ShiftKey AS LONG

  ' check if either shift key is pressed
  ShiftKey = (fg_kbtest(42) = 1) OR (fg_kbtest(54) = 1)

  ' + and - move cube along the z axis (+ is toward viewer, - is
  ' away from viewer)
  IF fg_kbtest(74) = 1 THEN
    zWorld = zWorld + 3.0
    Redraw = %TRUE
  ELSEIF fg_kbtest(78) = 1 THEN
    zWorld = zWorld - 3.0
    Redraw = %TRUE

  ' left and right arrow keys move cube along x axis
  ELSEIF fg_kbtest(75) = 1 THEN
    xWorld = xWorld - 3.0
    Redraw = %TRUE
  ELSEIF fg_kbtest(77) = 1 THEN
    xWorld = xWorld + 3.0
    Redraw = %TRUE

  ' up and down arrow keys move cube along y axis
  ELSEIF fg_kbtest(72) = 1 THEN
    yWorld = yWorld + 3.0
    Redraw = %TRUE
  ELSEIF fg_kbtest(80) = 1 THEN
    yWorld = yWorld - 3.0
    Redraw = %TRUE

  ' x rotates counterclockwise around x axis, X rotates clockwise
  ELSEIF fg_kbtest(45) = 1 THEN
    IF ShiftKey THEN
      xAngle = xAngle + 6
      IF xAngle >= 360 THEN xAngle = xAngle - 360
    ELSE
      xAngle = xAngle - 6
      IF xAngle < 0 THEN xAngle = xAngle + 360
    END IF
    Redraw = %TRUE

  ' y rotates counterclockwise around y axis, Y rotates clockwise
  ELSEIF fg_kbtest(21) = 1 THEN
    IF ShiftKey THEN
      yAngle = yAngle + 6
      IF yAngle >= 360 THEN yAngle = yAngle - 360
    ELSE
      yAngle = yAngle - 6
      IF yAngle < 0 THEN yAngle = yAngle + 360
    END IF
    Redraw = %TRUE

  ' z rotates counterclockwise around z axis, Z rotates clockwise
  ELSEIF fg_kbtest(44) = 1 THEN
    IF ShiftKey THEN
      zAngle = zAngle + 6
      IF zAngle >= 360 THEN zAngle = zAngle - 360
    ELSE
      zAngle = zAngle - 6
      IF zAngle < 0 THEN zAngle = zAngle + 360
    END IF
    Redraw = %TRUE
  END IF

  ' if the cube's position or orientation changed, redraw the cube
  IF Redraw THEN
    ' erase the previous frame from the virtual buffer
    fg_setcolor -1
    fg_fillpage

    ' define the cube's new position and rotation in 3D world space
    fg_3Dsetobject xWorld, yWorld, zWorld, xAngle*10, yAngle*10, zAngle*10

    ' draw the cube
    CALL DrawCube

    ' display what we just drew
    fg_vbscale 0, %vbWidth-1, 0, %vbHeight-1, 0, cxClient-1, 0, cyClient-1
    Redraw = %FALSE
  END IF

END SUB

'*****************************************************************************
'                                                                            *
'  DrawCube()                                                                *
'                                                                            *
'  Draws each of the six cube faces in 3D world space.                       *
'                                                                            *
'*****************************************************************************

SUB DrawCube()

  LOCAL I AS LONG
  LOCAL R AS LONG, G AS LONG, B AS LONG

  FOR I = 1 TO 6
    IF vbDepth > 8 THEN
      fg_getrgb Colors(I), R, G, B
      fg_setcolorrgb R, G, B
    ELSE
      fg_setcolor Colors(I)
    END IF
    fg_3Dpolygonobject Faces(1,I).X, 4
  NEXT

END SUB

'*****************************************************************************
'                                                                            *
'  InitGlobals()                                                             *
'                                                                            *
'  Initialize global variables and arrays. Called from the WM_CREATE handler.*
'                                                                            *
'*****************************************************************************

SUB InitGlobals()

   Colors(1) = 84: Colors(2) = 88: Colors(3) = 92
   Colors(4) = 96: Colors(5) = 100: Colors(6) = 104

   Faces(1,1).X = 20.0: Faces(1,1).Y = -20.0: Faces(1,1).Z = -20.0
   Faces(2,1).X = -20.0: Faces(2,1).Y = -20.0: Faces(2,1).Z = -20.0
   Faces(3,1).X = -20.0: Faces(3,1).Y = 20.0: Faces(3,1).Z = -20.0
   Faces(4,1).X = 20.0: Faces(4,1).Y = 20.0: Faces(4,1).Z = -20.0

   Faces(1,2).X = -20.0: Faces(1,2).Y = -20.0: Faces(1,2).Z = -20.0
   Faces(2,2).X = -20.0: Faces(2,2).Y = -20.0: Faces(2,2).Z = 20.0
   Faces(3,2).X = -20.0: Faces(3,2).Y = 20.0: Faces(3,2).Z = 20.0
   Faces(4,2).X = -20.0: Faces(4,2).Y = 20.0: Faces(4,2).Z = -20.0

   Faces(1,3).X = 20.0: Faces(1,3).Y = 20.0: Faces(1,3).Z = 20.0
   Faces(2,3).X = -20.0: Faces(2,3).Y = 20.0: Faces(2,3).Z = 20.0
   Faces(3,3).X = -20.0: Faces(3,3).Y = -20.0: Faces(3,3).Z = 20.0
   Faces(4,3).X = 20.0: Faces(4,3).Y = -20.0: Faces(4,3).Z = 20.0

   Faces(1,4).X = 20.0: Faces(1,4).Y = -20.0: Faces(1,4).Z = 20.0
   Faces(2,4).X = 20.0: Faces(2,4).Y = -20.0: Faces(2,4).Z = -20.0
   Faces(3,4).X = 20.0: Faces(3,4).Y = 20.0: Faces(3,4).Z = -20.0
   Faces(4,4).X = 20.0: Faces(4,4).Y = 20.0: Faces(4,4).Z = 20.0

   Faces(1,5).X = 20.0: Faces(1,5).Y = -20.0: Faces(1,5).Z = 20.0
   Faces(2,5).X = -20.0: Faces(2,5).Y = -20.0: Faces(2,5).Z = 20.0
   Faces(3,5).X = -20.0: Faces(3,5).Y = -20.0: Faces(3,5).Z = -20.0
   Faces(4,5).X = 20.0: Faces(4,5).Y = -20.0: Faces(4,5).Z = -20.0

   Faces(1,6).X = 20.0: Faces(1,6).Y = 20.0: Faces(1,6).Z = -20.0
   Faces(2,6).X = -20.0: Faces(2,6).Y = 20.0: Faces(2,6).Z = -20.0
   Faces(3,6).X = -20.0: Faces(3,6).Y = 20.0: Faces(3,6).Z = 20.0
   Faces(4,6).X = 20.0: Faces(4,6).Y = 20.0: Faces(4,6).Z = 20.0

END SUB
