'*****************************************************************************
'                                                                            *
'  Dcb.bas                                                                   *
'                                                                            *
'  This program demonstrates the Fastgraph for Windows direct color bitmap   *
'  display functions.                                                        *
'                                                                            *
'*****************************************************************************

#COMPILE EXE "Dcb.exe"

#INCLUDE "FGWin.inc"
#RESOURCE "Dcb.pbr"

%vbWidth = 320
%vbHeight = 240

DECLARE FUNCTION AppPath() AS STRING

GLOBAL Bird() AS WORD

'------------------------------------------------------------------------------
' Menu identifiers much match those in Dcb.h
%IDM_DrawDCB = 1
%IDM_ClipDCB = 2
%IDM_RevDCB  = 3
%IDM_FlipDCB = 4
%IDM_PutDCB  = 5
%IDM_Exit    = 6

'------------------------------------------------------------------------------

FUNCTION WINMAIN (BYVAL hInstance     AS LONG, _
                  BYVAL hPrevInstance AS LONG, _
                  lpCmdLine           AS ASCIIZ PTR, _
                  BYVAL iCmdShow      AS LONG) AS LONG

  LOCAL Msg         AS tagMsg
  LOCAL wndclass    AS WndClassEx
  LOCAL szClassName AS ASCIIZ * 80
  LOCAL hWnd        AS LONG

  DIM Bird(1:40*20)

  szClassName            = "FGdcb"

  wndclass.cbSize        = SIZEOF(WndClass)
  wndclass.style         = %CS_HREDRAW OR %CS_VREDRAW OR %CS_OWNDC
  wndclass.lpfnWndProc   = CODEPTR(WindowProc)
  wndclass.cbClsExtra    = 0
  wndclass.cbWndExtra    = 0
  wndclass.hInstance     = hInstance
  wndclass.hIcon         = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  wndclass.hCursor       = LoadCursor(%NULL, BYVAL %IDC_ARROW)
  wndclass.hbrBackground = %NULL
  wndclass.lpszMenuName  = VARPTR(szClassName)
  wndclass.lpszClassName = VARPTR(szClassName)
  wndclass.hIconSm       = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  RegisterClassEx wndclass

  hWnd = CreateWindow( _
    szClassName, _               ' window class name
    "Direct Color Bitmap Demo", _' window caption
    %WS_OVERLAPPEDWINDOW, _      ' window style
    %CW_USEDEFAULT, _            ' initial x position
    %CW_USEDEFAULT, _            ' initial y position
    %CW_USEDEFAULT, _            ' initial x size
    %CW_USEDEFAULT, _            ' initial y size
    %NULL, _                     ' parent window handle
    %NULL, _                     ' window menu handle
    hInstance, _                 ' program instance handle
    BYVAL %NULL)                 ' creation parameters

  ShowWindow hWnd, iCmdShow
  UpdateWindow hWnd

  WHILE GetMessage(Msg, %NULL, 0, 0)
    TranslateMessage Msg
    DispatchMessage Msg
  WEND

  FUNCTION = Msg.wParam

END FUNCTION

'------------------------------------------------------------------------------

GLOBAL hDC AS LONG
GLOBAL hPal AS LONG
GLOBAL hVB AS LONG
GLOBAL cxClient AS LONG, cyClient AS LONG

FUNCTION WindowProc (BYVAL hWnd AS LONG, BYVAL iMsg AS LONG, _
                     BYVAL wParam AS LONG, BYVAL lParam AS LONG) EXPORT AS LONG

  LOCAL ps AS PAINTSTRUCT

  SELECT CASE iMsg

    CASE %WM_CREATE
      hDC = GetDC(hWnd)
      fg_setdc hDC
      hPal = fg_defpal()
      fg_realize hPal

      fg_vbinit
      fg_vbdepth 16
      hVB = fg_vballoc(%vbWidth, %vbHeight)
      fg_vbopen hVB
      fg_vbcolors

      fg_setcolorrgb 85, 255, 85
      fg_fillpage
      CALL LoadBird
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_COMMAND
      SELECT CASE wParam

        CASE %IDM_DrawDCB
          CALL DrawDCB_Click

        CASE %IDM_ClipDCB
          CALL ClipDCB_Click

        CASE %IDM_RevDCB
          CALL RevDCB_Click

        CASE %IDM_FlipDCB
          CALL FlipDCB_Click

        CASE %IDM_PutDCB
          CALL PutDCB_Click

        CASE %IDM_Exit
          DestroyWindow hWnd
      END SELECT
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_PAINT
      BeginPaint hWnd, ps
      fg_vbscale 0, %vbWidth-1, 0, %vbHeight-1, 0, cxClient-1, 0, cyClient-1
      EndPaint hWnd, ps
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SETFOCUS
      fg_realize hPal
      InvalidateRect hWnd, BYVAL %NULL, %TRUE
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SIZE
      cxClient = LOWRD(lParam)
      cyClient = HIWRD(lParam)
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_DESTROY
      fg_vbclose
      fg_vbfree hVB
      fg_vbfin
      DeleteObject hPal
      ReleaseDC hWnd, hDC
      PostQuitMessage 0
      FUNCTION = 0
      EXIT FUNCTION

  END SELECT

  FUNCTION = DefWindowProc(hWnd, iMsg, wParam, lParam)

END FUNCTION

SUB ClipDCB_Click()
  fg_fillpage
  fg_move -20, %vbHeight/2 + 10
  fg_clipdcb Bird(1), 40, 20
  fg_vbscale 0, %vbWidth-1, 0, %vbHeight-1, 0, cxClient-1, 0, cyClient-1
END SUB

SUB DrawDCB_Click()
  fg_fillpage
  fg_move(%vbWidth/2 - 20, %vbHeight/2 + 10
  fg_drawdcb Bird(1), 40, 20
  fg_vbscale 0, %vbWidth-1, 0, %vbHeight-1, 0, cxClient-1, 0, cyClient-1
END SUB

SUB FlipDCB_Click()
  fg_fillpage
  fg_move %vbWidth-20, %vbHeight/2 + 10
  fg_flipdcb Bird(1), 40, 20
  fg_vbscale 0, %vbWidth-1, 0, %vbHeight-1, 0, cxClient-1, 0, cyClient-1
END SUB

SUB PutDCB_Click()
  fg_fillpage
  fg_move %vbWidth/2 - 20, %vbHeight/2 + 10
  fg_putdcb Bird(1), 40, 20
  fg_vbscale 0, %vbWidth-1, 0, %vbHeight-1, 0, cxClient-1, 0, cyClient-1
END SUB

SUB RevDCB_Click()
  fg_fillpage
  fg_move %vbWidth/2 - 20, %vbHeight/2 + 10
  fg_revdcb Bird(1), 40, 20
  fg_vbscale 0, %vbWidth-1, 0, %vbHeight-1, 0, cxClient-1, 0, cyClient-1
END SUB

FUNCTION AppPath() AS STRING
  LOCAL ModulePath AS ASCIIZ * 256
  GetModuleFileName %NULL, ModulePath, SIZEOF(ModulePath)
  FUNCTION = LEFT$(ModulePath, INSTR(-1, ModulePath, "\")-1)
END FUNCTION

SUB LoadBird()
  LOCAL I AS LONG
  OPEN AppPath+"\BirdHC.dat" FOR INPUT ACCESS READ AS #1
  FOR I = 1 TO 40*20
    INPUT #1, Bird(I)
  NEXT I
  CLOSE #1
END SUB
