'*****************************************************************************
'                                                                            *
'  ImgProc.bas                                                               *
'                                                                            *
'  This program demonstrates several of the Fastgraph for Windows image      *
'  processing functions.                                                     *
'                                                                            *
'*****************************************************************************

#COMPILE EXE "ImgProc.exe"

#INCLUDE "FGWin.inc"
#INCLUDE "Comdlg32.inc"
#RESOURCE "ImgProc.pbr"

GLOBAL Context() AS BYTE
GLOBAL FileHeader() AS BYTE

'------------------------------------------------------------------------------
' Menu identifiers much match those in ImgProc.h

%IDM_File_Open                = 101
%IDM_File_SaveAs              = 102
%IDM_File_Details             = 103
%IDM_File_Exit                = 104

%IDM_Edit_Undo                = 201
%IDM_Edit_RestoreOriginal     = 202
%IDM_Edit_ContrastEnhancement = 203
%IDM_Edit_GammaCorrection     = 204
%IDM_Edit_Grayscale           = 205
%IDM_Edit_PhotoInversion      = 206

'------------------------------------------------------------------------------

FUNCTION WINMAIN (BYVAL hInstance     AS LONG, _
                  BYVAL hPrevInstance AS LONG, _
                  lpCmdLine           AS ASCIIZ PTR, _
                  BYVAL iCmdShow      AS LONG) AS LONG

  LOCAL Msg         AS tagMsg
  LOCAL wndclass    AS WndClassEx
  LOCAL szClassName AS ASCIIZ * 80
  LOCAL hWnd        AS LONG

  DIM Context(1:48)
  DIM FileHeader(1:128)

  szClassName            = "FGimgproc"

  wndclass.cbSize        = SIZEOF(WndClass)
  wndclass.style         = %CS_HREDRAW OR %CS_VREDRAW OR %CS_OWNDC
  wndclass.lpfnWndProc   = CODEPTR(WindowProc)
  wndclass.cbClsExtra    = 0
  wndclass.cbWndExtra    = 0
  wndclass.hInstance     = hInstance
  wndclass.hIcon         = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  wndclass.hCursor       = LoadCursor(%NULL, BYVAL %IDC_ARROW)
  wndclass.hbrBackground = %NULL
  wndclass.lpszMenuName  = VARPTR(szClassName)
  wndclass.lpszClassName = VARPTR(szClassName)
  wndclass.hIconSm       = LoadIcon(hInstance, BYVAL %IDI_APPLICATION)
  RegisterClassEx wndclass

  hWnd = CreateWindow( _
    szClassName, _               ' window class name
    "Image Processing Demo", _   ' window caption
    %WS_OVERLAPPEDWINDOW, _      ' window style
    %CW_USEDEFAULT, _            ' initial x position
    %CW_USEDEFAULT, _            ' initial y position
    %CW_USEDEFAULT, _            ' initial x size
    %CW_USEDEFAULT, _            ' initial y size
    %NULL, _                     ' parent window handle
    %NULL, _                     ' window menu handle
    hInstance, _                 ' program instance handle
    BYVAL %NULL)                 ' creation parameters

  ShowWindow hWnd, iCmdShow
  UpdateWindow hWnd

  WHILE GetMessage(Msg, %NULL, 0, 0)
    TranslateMessage Msg
    DispatchMessage Msg
  WEND

  FUNCTION = Msg.wParam

END FUNCTION

'------------------------------------------------------------------------------

GLOBAL hDC AS LONG
GLOBAL hMenu AS LONG
GLOBAL hPal AS LONG
GLOBAL hVB AS LONG, hVBoriginal AS LONG, hVBundo AS LONG
GLOBAL cxClient AS LONG, cyClient AS LONG
GLOBAL cxBuffer AS LONG, cyBuffer AS LONG

GLOBAL nColors AS LONG

GLOBAL DefaultExt AS STRING
GLOBAL FileSpec AS STRING
GLOBAL mbString AS STRING

FUNCTION WindowProc (BYVAL hWnd AS LONG, BYVAL iMsg AS LONG, _
                     BYVAL wParam AS LONG, BYVAL lParam AS LONG) EXPORT AS LONG

  LOCAL ps AS PAINTSTRUCT

  SELECT CASE iMsg

    CASE %WM_CREATE
      hDC = GetDC(hWnd)
      fg_setdc hDC
      hPal = fg_defpal()
      fg_realize hPal

      ' initialize the virtual buffer environment
      fg_vbinit
      fg_vbdepth 24

      ' create the main virtual buffer for the working copy of the image
      cxBuffer = 32
      cyBuffer = 32
      hVB = fg_vballoc(cxBuffer, cyBuffer)
      fg_vbopen hVB

      ' create two additional virtual buffers -- one for a copy of the
      ' original image, and one used for the undo operation
      hVBoriginal = fg_vballoc(cxBuffer, cyBuffer)
      hVBundo = fg_vballoc(cxBuffer, cyBuffer)

      ' start with a window full of white pixels
      fg_setcolor -1
      fg_fillpage

      hMenu = GetMenu(hWnd)
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_COMMAND
      SELECT CASE wParam

        CASE %IDM_File_Open
          CALL Open_Click

        CASE %IDM_File_SaveAs
          CALL SaveAs_Click

        CASE %IDM_File_Details
          CALL Details_Click

        CASE %IDM_File_Exit
          DestroyWindow hWnd

        CASE %IDM_Edit_Undo
          CALL Undo_Click

        CASE %IDM_Edit_RestoreOriginal
          CALL RestoreOriginal_Click

        CASE %IDM_Edit_ContrastEnhancement
          CALL ContrastEnhancement_Click

        CASE %IDM_Edit_GammaCorrection
          CALL GammaCorrection_Click

        CASE %IDM_Edit_Grayscale
          CALL Grayscale_Click

        CASE %IDM_Edit_PhotoInversion
          CALL PhotoInversion_Click
      END SELECT
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_PAINT
      BeginPaint hWnd, ps
      fg_vbscale 0, cxBuffer-1, 0, cyBuffer-1, 0, cxClient-1, 0, cyClient-1
      EndPaint hWnd, ps
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SETFOCUS
      fg_realize hPal
      InvalidateRect hWnd, BYVAL %NULL, %TRUE
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_SIZE
      cxClient = LOWRD(lParam)
      cyClient = HIWRD(lParam)
      FUNCTION = 0
      EXIT FUNCTION

    CASE %WM_DESTROY
      fg_vbclose
      fg_vbfree hVB
      fg_vbfree hVBoriginal
      fg_vbfree hVBundo
      fg_vbfin
      DeleteObject hPal
      ReleaseDC hWnd, hDC
      PostQuitMessage 0
      FUNCTION = 0
      EXIT FUNCTION

  END SELECT

  FUNCTION = DefWindowProc(hWnd, iMsg, wParam, lParam)

END FUNCTION

'*****************************************************************************
'                                                                            *
'  Event handlers for the items on the File menu.                            *
'                                                                            *
'*****************************************************************************

SUB Open_Click()

  ' open the bmp, jpeg, or pcx image file
  FileSpec = ""
  IF OpenFileDialog(GetActiveWindow(), "Open File", FileSpec, "", _
    "All image files (*.bmp,*.jpg,*.pcx)|*.BMP;*.JPG;*.PCX|" _
    + "BMP files (*.bmp)|*.BMP|" _
    + "JPEG files (*.jpg)|*.JPG|" _
    + "PCX files (*.pcx)|*.PCX|", _
    "", %OFN_READONLY) = 0 THEN EXIT SUB

  ' check for a bmp file
  IF fg_bmphead(FileSpec, FileHeader(1)) = 0 THEN
    MOUSEPTR 11
    nColors = fg_bmppal(FileSpec, BYVAL %NULL)
    fg_bmpsize FileHeader(1), cxBuffer, cyBuffer
    CALL SwitchBuffers
    fg_showbmp FileSpec, 0
    DefaultExt = "bmp"

  ' check for a jpeg file
  ELSEIF fg_jpeghead(FileSpec, FileHeader(1)) = 0 THEN
    MOUSEPTR 11
    nColors = 0
    fg_jpegsize FileHeader(1), cxBuffer, cyBuffer
    CALL SwitchBuffers
    fg_showjpeg FileSpec, 0
    DefaultExt = "pcx"

  ' check for a pcx file
  ELSEIF fg_pcxhead(FileSpec, FileHeader(1)) = 0 THEN
    MOUSEPTR 11
    nColors = fg_pcxpal(FileSpec, BYVAL %NULL)
    fg_pcxsize FileHeader(1), cxBuffer, cyBuffer
    CALL SwitchBuffers
    fg_move 0, 0
    fg_showpcx FileSpec, %FG_AT_XY
    DefaultExt = "pcx"

  ' the file is not a valid bmp, jpeg, or pcx file
  ELSE
    mbString = FileSpec + CHR$(13) + "is not a recognized image file."
    MSGBOX mbString, %MB_ICONSTOP OR %MB_OK, "Error"
    EXIT SUB
  END IF

  ' make a copy of the original image
  fg_copypage hVB, hVBoriginal

  ' display the image
  fg_vbscale 0, cxBuffer-1, 0, cyBuffer-1, 0, cxClient-1, 0, cyClient-1
  MOUSEPTR 0

  ' enable remaining items on the File menu, and the image processing
  ' items on the Edit menu
  EnableMenuItem hMenu, %IDM_File_SaveAs, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_File_Details, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_Edit_Undo, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_Edit_RestoreOriginal, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_Edit_ContrastEnhancement, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_Edit_GammaCorrection, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_Edit_Grayscale, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_Edit_PhotoInversion, %MF_ENABLED

END SUB

'------------------------------------------------------------------------------

SUB SaveAs_Click()

  ' save image as a bmp file (original image was bmp)
  IF DefaultExt = "bmp" THEN
    FileSpec = LEFT$(FileSpec, INSTR(FileSpec, ".")-1) + ".bmp"
    IF SaveFileDialog(GetActiveWindow(), "Save File", FileSpec, "", _
      "BMP files (*.bmp)|*.BMP", "bmp", _
      %OFN_HIDEREADONLY OR %OFN_OVERWRITEPROMPT OR %OFN_PATHMUSTEXIST) = 0 THEN EXIT SUB
    MOUSEPTR 11
    fg_makebmp 0, cxBuffer-1, 0, cyBuffer-1, 24, FileSpec
    nColors = 0
    MOUSEPTR 0

  ' save image as a pcx file (original image was jpeg or pcx)
  ELSEIF DefaultExt = "pcx" THEN
    FileSpec = LEFT$(FileSpec, INSTR(FileSpec, ".")-1) + ".pcx"
    IF SaveFileDialog(GetActiveWindow(), "Save File", FileSpec, "", _
      "PCX files (*.pcx)|*.PCX", "pcx", _
      %OFN_HIDEREADONLY OR %OFN_OVERWRITEPROMPT OR %OFN_PATHMUSTEXIST) = 0 THEN EXIT SUB
    MOUSEPTR 11
    fg_makepcx 0, cxBuffer-1, 0, cyBuffer-1, FileSpec
    nColors = 0
    MOUSEPTR 0
  END IF

END SUB

'------------------------------------------------------------------------------

SUB Details_Click()

  mbString = FileSpec + CHR$(13) + _
             STR$(cxBuffer) + " x" + STR$(cyBuffer) + " pixels" + CHR$(13)
  IF nColors > 0 THEN
    mbString = mbString + STR$(nColors) + " colors"
  ELSE
    mbString = mbString + "24-bit RGB"
  END IF
  MSGBOX mbString, %MB_ICONINFORMATION OR %MB_OK, "Information"

END SUB

'*****************************************************************************
'                                                                            *
'  Event handlers for the items on the Edit menu.                            *
'                                                                            *
'*****************************************************************************

SUB Undo_Click()

  ' undo the previous image processing operation
  fg_copypage hVBundo, hVB
  fg_vbscale 0, cxBuffer-1, 0, cyBuffer-1, 0, cxClient-1, 0, cyClient-1
  EnableMenuItem hMenu, %IDM_Edit_Undo, %MF_GRAYED
  EnableMenuItem hMenu, %IDM_Edit_RestoreOriginal, %MF_ENABLED

END SUB

'------------------------------------------------------------------------------

SUB RestoreOriginal_Click()

  ' restore the original image
  fg_copypage hVB, hVBundo
  fg_copypage hVBoriginal, hVB
  fg_vbscale 0, cxBuffer-1, 0, cyBuffer-1, 0, cxClient-1, 0, cyClient-1
  EnableMenuItem hMenu, %IDM_Edit_Undo, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_Edit_RestoreOriginal, %MF_GRAYED

END SUB

'------------------------------------------------------------------------------

SUB ContrastEnhancement_Click()

  ' perform a contrast enhancement transform on the active virtual buffer
  fg_copypage hVB, hVBundo
  fg_move 0, cyBuffer-1
  fg_contvb 63, 192, cxBuffer, cyBuffer
  fg_vbscale 0, cxBuffer-1, 0, cyBuffer-1, 0, cxClient-1, 0, cyClient-1
  EnableMenuItem hMenu, %IDM_Edit_Undo, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_Edit_RestoreOriginal, %MF_ENABLED

END SUB

'------------------------------------------------------------------------------

SUB GammaCorrection_Click()

  ' perform a gamma correction transform on the active virtual buffer
  fg_copypage hVB, hVBundo
  fg_move 0, cyBuffer-1
  fg_gammavb 0.45, cxBuffer, cyBuffer
  fg_vbscale 0, cxBuffer-1, 0, cyBuffer-1, 0, cxClient-1, 0, cyClient-1
  EnableMenuItem hMenu, %IDM_Edit_Undo, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_Edit_RestoreOriginal, %MF_ENABLED

END SUB

'------------------------------------------------------------------------------

SUB Grayscale_Click()

  ' perform a grayscale transform on the active virtual buffer
  fg_copypage hVB, hVBundo
  fg_move 0, cyBuffer-1
  fg_grayvb cxBuffer, cyBuffer
  fg_vbscale 0, cxBuffer-1, 0, cyBuffer-1, 0, cxClient-1, 0, cyClient-1
  EnableMenuItem hMenu, %IDM_Edit_Undo, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_Edit_RestoreOriginal, %MF_ENABLED

END SUB

'------------------------------------------------------------------------------

SUB PhotoInversion_Click()

  ' perform a photo-inversion transform on the active virtual buffer
  fg_copypage hVB, hVBundo
  fg_move 0, cyBuffer-1
  fg_photovb cxBuffer, cyBuffer
  fg_vbscale 0, cxBuffer-1, 0, cyBuffer-1, 0, cxClient-1, 0, cyClient-1
  EnableMenuItem hMenu, %IDM_Edit_Undo, %MF_ENABLED
  EnableMenuItem hMenu, %IDM_Edit_RestoreOriginal, %MF_ENABLED

END SUB

'*****************************************************************************
'                                                                            *
'  SwitchBuffers()                                                           *
'                                                                            *
'  Close the and release the virtual buffers for the current image, then     *
'  create and open new virtual buffers for the new image file.               *
'                                                                            *
'*****************************************************************************

SUB SwitchBuffers()

  fg_vbclose
  fg_vbfree hVB
  fg_vbfree hVBoriginal
  fg_vbfree hVBundo
  hVB = fg_vballoc(cxBuffer, cyBuffer)
  fg_vbopen hVB
  hVBoriginal = fg_vballoc(cxBuffer, cyBuffer)
  hVBundo = fg_vballoc(cxBuffer, cyBuffer)

END SUB
