// ChildView.cpp : implementation of the CChildView class
//

#include "stdafx.h"
#include "ImgProc.h"

#include "ChildView.h"
#include "MainFrame.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CChildView

CChildView::CChildView()
{
	m_hDC = NULL;
	m_hPal = NULL;
	m_hVB = -1;
	m_cxClient = 0;
	m_cyClient = 0;
	m_cxBuffer = 32;
	m_cyBuffer = 32;
	m_nColors = 0;
}

CChildView::~CChildView()
{
	if (m_hVB >= 0)
	{
		fg_vbclose();
		fg_vbfree(m_hVB);
		fg_vbfree(m_hVBoriginal);
		fg_vbfree(m_hVBundo);
	}
	fg_vbfin();
	if (m_hPal)
	{
		DeleteObject(m_hPal);
		m_hPal = NULL;
	}
	if (m_hDC)
	{
		::ReleaseDC(m_hWnd, m_hDC);
		m_hDC = NULL;
	}
}

BEGIN_MESSAGE_MAP(CChildView, CWnd)
	//{{AFX_MSG_MAP(CChildView)
	ON_WM_PAINT()
	ON_WM_SIZE()
	ON_WM_SETFOCUS()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CChildView message handlers

BOOL CChildView::PreCreateWindow(CREATESTRUCT& cs)
{
	if (!CWnd::PreCreateWindow(cs))
		return FALSE;

	cs.dwExStyle |= WS_EX_CLIENTEDGE;
	cs.style &= ~WS_BORDER;
	cs.lpszClass = AfxRegisterWndClass(CS_OWNDC|CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS,
		LoadCursor(NULL,IDC_ARROW), NULL, NULL);
	return TRUE;
}

BOOL CChildView::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext)
{
	BOOL bRet = CWnd::Create(lpszClassName, lpszWindowName, dwStyle, rect, pParentWnd, nID, pContext);
	if (bRet)
	{
		m_hDC = ::GetDC(m_hWnd);
		fg_setdc(m_hDC);
		m_hPal = fg_defpal();
		fg_realize(m_hPal);

		// initialize the virtual buffer environment
		fg_vbinit();
		fg_vbdepth(24);

		// create the main virtual buffer for the working copy of the image
		m_hVB = fg_vballoc(m_cxBuffer,m_cyBuffer);
		fg_vbopen(m_hVB);

		// create two additional virtual buffers -- one for a copy of the
		// original image, and one used for the undo operation
		m_hVBoriginal = fg_vballoc(m_cxBuffer,m_cyBuffer);
		m_hVBundo = fg_vballoc(m_cxBuffer,m_cyBuffer);

		// start with a window full of white pixels
		fg_setcolor(-1);
		fg_fillpage();
	}
	return bRet;
}

void CChildView::OnPaint()
{
	CPaintDC dc(this); // device context for painting

	fg_vbscale(0,m_cxBuffer-1,0,m_cyBuffer-1,0,m_cxClient-1,0,m_cyClient-1);
}

void CChildView::OnSize(UINT, int cx, int cy)
{
	m_cxClient = cx;
	m_cyClient = cy;
}

void CChildView::OnSetFocus(CWnd* pOldWnd)
{
	CWnd::OnSetFocus(pOldWnd);
	fg_realize(m_hPal);
	Invalidate();
}

/****************************************************************************\
*                                                                            *
*  Event handlers for the items on the File menu.                            *
*                                                                            *
\****************************************************************************/

BOOL CChildView::FileOpenClick()
{
	// open the bmp, jpeg, or pcx image file
	CFileDialog dlg(TRUE,NULL,NULL,OFN_READONLY,
		"All image files (*.bmp,*.jpg,*.pcx)|*.BMP;*.JPG;*.PCX|" \
		"BMP files (*.bmp)|*.BMP|" \
		"JPEG files (*.jpg)|*.JPG|" \
		"PCX files (*.pcx)|*.PCX||",
      this);
	if (dlg.DoModal() != IDOK) return FALSE;
	m_FileName = dlg.GetPathName();

	// check for a bmp file
	if (fg_bmphead((LPSTR)((LPCSTR)m_FileName),m_FileHeader) == 0)
	{
		CWaitCursor wait;
		m_nColors = fg_bmppal((LPSTR)((LPCSTR)m_FileName),NULL);
		fg_bmpsize(m_FileHeader,&m_cxBuffer,&m_cyBuffer);
		SwitchBuffers();
		fg_showbmp((LPSTR)((LPCSTR)m_FileName),0);
		m_DefaultExt = "bmp";
	}

	// check for a jpeg file
	else if (fg_jpeghead((LPSTR)((LPCSTR)m_FileName),m_FileHeader) == 0)
	{
		CWaitCursor wait;
		m_nColors = 0;
		fg_jpegsize(m_FileHeader,&m_cxBuffer,&m_cyBuffer);
		SwitchBuffers();
		fg_showjpeg((LPSTR)((LPCSTR)m_FileName),0);
		m_DefaultExt = "pcx";
	}

	// check for a pcx file
	else if (fg_pcxhead((LPSTR)((LPCSTR)m_FileName),m_FileHeader) == 0)
	{
		CWaitCursor wait;
		m_nColors = fg_pcxpal((LPSTR)((LPCSTR)m_FileName),NULL);
		fg_pcxsize(m_FileHeader,&m_cxBuffer,&m_cyBuffer);
		SwitchBuffers();
		fg_move(0,0);
		fg_showpcx((LPSTR)((LPCSTR)m_FileName),FG_AT_XY);
		m_DefaultExt = "pcx";
	}

	// the file is not a valid bmp, jpeg, or pcx file
	else
	{
		CString mbString = m_FileName + " is not a recognized image file.";
		MessageBox(mbString,"Error",MB_ICONSTOP|MB_OK);
		return FALSE;
	}

	// make a copy of the original image
	fg_copypage(m_hVB,m_hVBoriginal);

	// display the image
	fg_vbscale(0,m_cxBuffer-1,0,m_cyBuffer-1,0,m_cxClient-1,0,m_cyClient-1);
	return TRUE;
}

void CChildView::FileSaveAsClick()
{
	// save image as a bmp file (original image was bmp)
	if (m_DefaultExt == "bmp")
	{
		lstrcpy(strrchr((LPSTR)((LPCSTR)m_FileName),'.'),".bmp");
		CFileDialog dlg(FALSE,"bmp",m_FileName,
			OFN_HIDEREADONLY|OFN_OVERWRITEPROMPT|OFN_PATHMUSTEXIST,
			"BMP files (*.bmp)|*.BMP||",this);
		if (dlg.DoModal() != IDOK) return;

		CWaitCursor wait;
		m_FileName = dlg.GetPathName();
		fg_makebmp(0,m_cxBuffer-1,0,m_cyBuffer-1,24,(LPSTR)((LPCSTR)m_FileName));
		m_nColors = 0;
	}

	// save image as a pcx file (original image was jpeg or pcx)
	else if (m_DefaultExt == "pcx")
	{
		lstrcpy(strrchr((LPSTR)((LPCSTR)m_FileName),'.'),".pcx");
		CFileDialog dlg(FALSE,"pcx",m_FileName,
			OFN_HIDEREADONLY|OFN_OVERWRITEPROMPT|OFN_PATHMUSTEXIST,
			"PCX files (*.pcx)|*.PCX||",this);
		if (dlg.DoModal() != IDOK) return;

		CWaitCursor wait;
		m_FileName = dlg.GetPathName();
		fg_makepcx(0,m_cxBuffer-1,0,m_cyBuffer-1,(LPSTR)((LPCSTR)m_FileName));
		m_nColors = 0;
	}
}

void CChildView::FileDetailsClick()
{
	CString mbString;

	// display the original image resolution and color depth
	mbString = m_FileName + "\n" +
		IntToStr(m_cxBuffer) + "x" + IntToStr(m_cyBuffer) + " pixels\n";
	if (m_nColors > 0)
		mbString += IntToStr(m_nColors) + " colors";
	else
		mbString += "24-bit RGB";
	MessageBox(mbString,"Information",MB_ICONINFORMATION|MB_OK);
}

/****************************************************************************\
*                                                                            *
*  Event handlers for the items on the Edit menu.                            *
*                                                                            *
\****************************************************************************/

void CChildView::EditUndoClick()
{
	// undo the previous image processing operation
	fg_copypage(m_hVBundo,m_hVB);
	fg_vbscale(0,m_cxBuffer-1,0,m_cyBuffer-1,0,m_cxClient-1,0,m_cyClient-1);
}

void CChildView::EditRestoreOriginalClick()
{
	// restore the original image
	fg_copypage(m_hVB,m_hVBundo);
	fg_copypage(m_hVBoriginal,m_hVB);
	fg_vbscale(0,m_cxBuffer-1,0,m_cyBuffer-1,0,m_cxClient-1,0,m_cyClient-1);
}

void CChildView::EditContrastEnhancementClick()
{
	// perform a contrast enhancement transform on the active virtual buffer
	fg_copypage(m_hVB,m_hVBundo);
	fg_move(0,m_cyBuffer-1);
	fg_contvb(63,192,m_cxBuffer,m_cyBuffer);
	fg_vbscale(0,m_cxBuffer-1,0,m_cyBuffer-1,0,m_cxClient-1,0,m_cyClient-1);
}

void CChildView::EditGammaCorrectionClick()
{
	// perform a gamma correction transform on the active virtual buffer
	fg_copypage(m_hVB,m_hVBundo);
	fg_move(0,m_cyBuffer-1);
	fg_gammavb(0.45,m_cxBuffer,m_cyBuffer);
	fg_vbscale(0,m_cxBuffer-1,0,m_cyBuffer-1,0,m_cxClient-1,0,m_cyClient-1);
}

void CChildView::EditGrayscaleClick()
{
	// perform a grayscale transform on the active virtual buffer
	fg_copypage(m_hVB,m_hVBundo);
	fg_move(0,m_cyBuffer-1);
	fg_grayvb(m_cxBuffer,m_cyBuffer);
	fg_vbscale(0,m_cxBuffer-1,0,m_cyBuffer-1,0,m_cxClient-1,0,m_cyClient-1);
}

void CChildView::EditPhotoInversionClick()
{
	// perform a photo-inversion transform on the active virtual buffer
	fg_copypage(m_hVB,m_hVBundo);
	fg_move(0,m_cyBuffer-1);
	fg_photovb(m_cxBuffer,m_cyBuffer);
	fg_vbscale(0,m_cxBuffer-1,0,m_cyBuffer-1,0,m_cxClient-1,0,m_cyClient-1);
}

/****************************************************************************\
*                                                                            *
*  IntToStr()                                                                *
*                                                                            *
*  Convert an integer quantity to a CString string.                          *
*                                                                            *
\****************************************************************************/

CString CChildView::IntToStr(int n)
{
	CString Result;

	Result.Format("%d",n);
	return Result;
}

/****************************************************************************\
*                                                                            *
*  SwitchBuffers()                                                           *
*                                                                            *
*  Close the and release the virtual buffers for the current image, then     *
*  create and open new virtual buffers for the new image file.               *
*                                                                            *
\****************************************************************************/

void CChildView::SwitchBuffers()
{
	fg_vbclose();
	fg_vbfree(m_hVB);
	fg_vbfree(m_hVBoriginal);
	fg_vbfree(m_hVBundo);
	m_hVB = fg_vballoc(m_cxBuffer,m_cyBuffer);
	fg_vbopen(m_hVB);
	m_hVBoriginal = fg_vballoc(m_cxBuffer,m_cyBuffer);
	m_hVBundo = fg_vballoc(m_cxBuffer,m_cyBuffer);
}
